@@ -22,7 +22,6 @@
 #include "config.h"
 
 #include "gcab-priv.h"
-#include <glib/gi18n-lib.h>
 
 static voidpf
 zalloc (voidpf opaque, uInt items, uInt size)
@@ -39,7 +38,7 @@ static gboolean
 cdata_set (cdata_t *cd, int type, guint8 *data, size_t size)
 {
     if (type > GCAB_COMPRESSION_MSZIP) {
-        g_critical (_("unsupported compression method %d"), type);
+        g_critical ("unsupported compression method %d", type);
         return FALSE;
     }
 
@@ -311,6 +310,19 @@ end:
     return success;
 }
 
+void
+cheader_free (cheader_t *ch)
+{
+    if (ch == NULL)
+        return;
+    g_free (ch->reserved);
+    g_free (ch->cab_prev);
+    g_free (ch->disk_prev);
+    g_free (ch->cab_next);
+    g_free (ch->disk_next);
+    g_free (ch);
+}
+
 G_GNUC_INTERNAL gboolean
 cfolder_write (cfolder_t *cf, GDataOutputStream *out,
                GCancellable *cancellable, GError **error)
@@ -350,6 +362,15 @@ end:
     return success;
 }
 
+void
+cfolder_free (cfolder_t *cf)
+{
+    if (cf == NULL)
+        return;
+    g_free (cf->reserved);
+    g_free (cf);
+}
+
 G_GNUC_INTERNAL gboolean
 cfile_write (cfile_t *cf, GDataOutputStream *out,
              GCancellable *cancellable, GError **error)
@@ -398,6 +419,15 @@ end:
     return success;
 }
 
+void
+cfile_free (cfile_t *cf)
+{
+    if (cf == NULL)
+        return;
+    g_free (cf->name);
+    g_free (cf);
+}
+
 static guint32
 compute_checksum (guint8 *in, guint16 ncbytes, guint32 seed)
 {
@@ -466,20 +496,31 @@ cdata_write (cdata_t *cd, GDataOutputStream *out, int type,
 }
 
 G_GNUC_INTERNAL void
-cdata_finish (cdata_t *cd, GError **error)
+cdata_free (cdata_t *cd)
 {
     z_stream *z = &cd->z;
-    int zret;
 
-    if (!z->opaque)
-        return;
+    if (cd->decomp.comptype == GCAB_COMPRESSION_LZX) {
+        LZXfdi_clear (&cd->decomp);
+    }
 
-    zret = inflateEnd (z);
-    z->opaque = NULL;
+    if (cd->decomp.comptype == GCAB_COMPRESSION_MSZIP) {
+        if (z->opaque) {
+            inflateEnd (z);
+            z->opaque = NULL;
+        }
+    }
+    g_free (cd->reserved);
+    g_free (cd);
+}
 
-    if (zret != Z_OK)
-        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
-                     "zlib failed: %s", zError (zret));
+static gint
+_enforce_checksum (void)
+{
+    static gint enforce = -1;
+    if (enforce == -1)
+        enforce = g_getenv ("GCAB_SKIP_CHECKSUM") == NULL ? 1 : 0;
+    return enforce;
 }
 
 G_GNUC_INTERNAL gboolean
@@ -493,6 +534,7 @@ cdata_read (cdata_t *cd, guint8 res_data, gint comptype,
     gsize buf_sz;
     guint8 *buf = NULL;
     guint32 datacsum;
+    guint32 checksum_tmp;
     guint8 sizecsum[4];
     guint16 nbytes_le;
 
@@ -508,8 +550,8 @@ cdata_read (cdata_t *cd, guint8 res_data, gint comptype,
         buf_sz = sizeof(cd->in);
         break;
     default:
-        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
-                     _("unsupported compression method %d"), compression);
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_NOT_SUPPORTED,
+                     "unsupported compression method %d", compression);
         break;
     }
     if (buf == NULL)
@@ -518,14 +560,20 @@ cdata_read (cdata_t *cd, guint8 res_data, gint comptype,
     R4 (cd->checksum);
     R2 (cd->ncbytes);
     if (cd->ncbytes > buf_sz) {
-        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_INVALID_DATA,
                      "tried to decompress %" G_GUINT16_FORMAT " bytes "
                      "into buffer of size %" G_GSIZE_FORMAT,
                      cd->ncbytes, buf_sz);
         return FALSE;
     }
     R2 (cd->nubytes);
-    cd->reserved = g_malloc (res_data);
+    if (cd->nubytes > CAB_BLOCKMAX) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_INVALID_DATA,
+                     "CDATA block of %" G_GUINT16_FORMAT " bytes "
+                     "was bigger than maximum size %i",
+                     cd->nubytes, CAB_BLOCKMAX);
+        return FALSE;
+    }
     RN (cd->reserved, res_data);
     RN (buf, cd->ncbytes);
 
@@ -534,10 +582,15 @@ cdata_read (cdata_t *cd, guint8 res_data, gint comptype,
     memcpy (&sizecsum[0], &nbytes_le, 2);
     nbytes_le = GUINT16_TO_LE (cd->nubytes);
     memcpy (&sizecsum[2], &nbytes_le, 2);
-    if (cd->checksum != compute_checksum (sizecsum, sizeof(sizecsum), datacsum)) {
-        g_set_error_literal (error, GCAB_ERROR, GCAB_ERROR_FAILED,
-                             _("incorrect checksum detected"));
-        return FALSE;
+    checksum_tmp = compute_checksum (sizecsum, sizeof(sizecsum), datacsum);
+    if (cd->checksum != checksum_tmp) {
+        if (_enforce_checksum ()) {
+            g_set_error_literal (error, GCAB_ERROR, GCAB_ERROR_INVALID_DATA,
+                                 "incorrect checksum detected");
+            return FALSE;
+        }
+        if (g_getenv ("GCAB_DEBUG"))
+            g_debug ("CDATA checksum 0x%08x", (guint) checksum_tmp);
     }
 
     if (g_getenv ("GCAB_DEBUG")) {
@@ -557,6 +610,7 @@ cdata_read (cdata_t *cd, guint8 res_data, gint comptype,
             cd->decomp.fdi = &cd->fdi;
             cd->decomp.inbuf = cd->in;
             cd->decomp.outbuf = cd->out;
+            cd->decomp.comptype = compression;
 
             ret = LZXfdi_init((comptype >> 8) & 0x1f, &cd->decomp);
             if (ret < 0)
@@ -572,6 +626,7 @@ cdata_read (cdata_t *cd, guint8 res_data, gint comptype,
         if (cd->in[0] != 'C' || cd->in[1] != 'K')
             goto end;
 
+        cd->decomp.comptype = compression;
         z_stream *z = &cd->z;
 
         z->avail_in = cd->ncbytes - 2;
