@@ -1,6 +1,7 @@
 /*
  * LibGCab
  * Copyright (c) 2012, Marc-André Lureau <marcandre.lureau@gmail.com>
+ * Copyright (c) 2017, Richard Hughes <richard@hughsie.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,18 +36,12 @@
  * extracting and creation of archives.
  */
 
-struct _GCabCabinetClass
-{
-    GObjectClass parent_class;
-};
-
-struct _GCabCabinet
-{
+struct _GCabCabinet {
     GObject parent_instance;
 
     GPtrArray *folders;
     GByteArray *reserved;
-    cheader_t cheader;
+    cheader_t *cheader;
     GByteArray *signature;
     GInputStream *stream;
 };
@@ -77,12 +72,14 @@ gcab_cabinet_finalize (GObject *object)
 {
     GCabCabinet *self = GCAB_CABINET (object);
 
+    cheader_free (self->cheader);
     g_ptr_array_unref (self->folders);
     if (self->reserved)
         g_byte_array_unref (self->reserved);
     if (self->signature)
         g_byte_array_unref (self->signature);
-    g_clear_object (&self->stream);
+    if (self->stream)
+        g_object_unref (self->stream);
 
     G_OBJECT_CLASS (gcab_cabinet_parent_class)->finalize (object);
 }
@@ -203,6 +200,24 @@ gcab_cabinet_get_folders (GCabCabinet *self)
     return self->folders;
 }
 
+/**
+ * gcab_cabinet_get_size:
+ * @cabinet:a #GCabCabinet
+ *
+ * Get the size of the compressed cabinet file.
+ *
+ * Returns: size in bytes
+ *
+ * Since: 1.0
+ **/
+guint32
+gcab_cabinet_get_size (GCabCabinet *self)
+{
+    if (self->cheader == NULL)
+        return 0;
+    return self->cheader->size;
+}
+
 /**
  * gcab_cabinet_write:
  * @cabinet: a #GCabCabinet
@@ -227,10 +242,7 @@ gcab_cabinet_write (GCabCabinet *self,
                     GCancellable *cancellable,
                     GError **error)
 {
-    cheader_t header = {
-        .offsetfiles = CFI_START, // CFHEADER + 1 * CFFOLDER
-        .nfolders = 1, // a single CAB folder is enough
-    };
+    g_autoptr(cheader_t) cheader = g_new0 (cheader_t, 1);
     cfolder_t folder = { 0, };
 
     g_return_val_if_fail (GCAB_IS_CABINET (self), FALSE);
@@ -238,113 +250,127 @@ gcab_cabinet_write (GCabCabinet *self,
     g_return_val_if_fail (G_IS_SEEKABLE (out), FALSE);
     g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), FALSE);
     g_return_val_if_fail (!error || *error == NULL, FALSE);
-    /* FIXME: current limitation of 1 folder */
-    g_return_val_if_fail (self->folders->len == 1, FALSE);
+
+    /* FIXME: support more than one folder */
+    cheader->offsetfiles = CFI_START; // CFHEADER + 1 * CFFOLDER
+    cheader->nfolders = 1; // a single CAB folder is enough
+
+    /* nothing to do */
+    if (self->folders->len == 0) {
+        g_set_error_literal (error, GCAB_ERROR, GCAB_ERROR_FAILED,
+                             "Cabinet has no added folders");
+        return FALSE;
+    }
+
+    /* unsupported */
+    if (self->folders->len > 1) {
+        g_set_error_literal (error, GCAB_ERROR, GCAB_ERROR_NOT_SUPPORTED,
+                             "Cabinet has more than one added folder");
+        return FALSE;
+    }
 
     GCabFolder *cabfolder = g_ptr_array_index (self->folders, 0);
-    GCabFile *file;
     gsize nfiles = gcab_folder_get_nfiles (cabfolder);
-    GInputStream *in = NULL;
-    GDataOutputStream *dstream = NULL;
-    gboolean success = FALSE;
+    g_autoptr(GDataOutputStream) dstream = NULL;
     gssize len, offset = 0;
     cdata_t block = { 0, };
     guint8 data[DATABLOCKSIZE];
     gsize written;
     size_t sumstr = 0;
-    GSList *l, *files;
-    cfile_t *prevf = NULL;
+    g_autoptr(GSList) files = NULL;
+    GCabFile *prevf = NULL;
 
     dstream = g_data_output_stream_new (out);
     g_data_output_stream_set_byte_order (dstream, G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN);
 
     if (self->reserved) {
-        header.offsetfiles += self->reserved->len + 4;
-        header.flags = CABINET_HEADER_RESERVE;
-        header.res_header = self->reserved->len;
-        header.res_folder = 0;
-        header.res_data = 0;
-        header.reserved = self->reserved->data;
+        cheader->offsetfiles += self->reserved->len + 4;
+        cheader->flags = CABINET_HEADER_RESERVE;
+        cheader->res_header = self->reserved->len;
+        cheader->res_folder = 0;
+        cheader->res_data = 0;
+        cheader->reserved = self->reserved->data;
     }
 
     files = gcab_folder_get_files (cabfolder);
-    for (l = files; l != NULL; l = l->next)
-        sumstr += strlen (GCAB_FILE (l->data)->name) + 1;
+    for (GSList *l = files; l != NULL; l = l->next) {
+        GCabFile *cabfile = GCAB_FILE (l->data);
+        sumstr += strlen (gcab_file_get_name (cabfile)) + 1;
+    }
 
-    folder.typecomp = cabfolder->comptype;
-    folder.offsetdata = header.offsetfiles + nfiles * 16 + sumstr;
+    folder.typecomp = gcab_folder_get_comptype (cabfolder);
+    folder.offsetdata = cheader->offsetfiles + nfiles * 16 + sumstr;
     folder.ndatab = gcab_folder_get_ndatablocks (cabfolder);
 
     /* avoid seeking to allow growing output streams */
     for (guint i = 0; i < folder.offsetdata; i++)
         if (!g_data_output_stream_put_byte (dstream, 0, cancellable, error))
-            goto end;
+            return FALSE;
 
-    for (l = files; l != NULL; l = l->next) {
-        file = GCAB_FILE (l->data);
+    for (GSList *l = files; l != NULL; l = l->next) {
+        g_autoptr(GInputStream) in = NULL;
+
+        GCabFile *file = GCAB_FILE (l->data);
         if (file_callback)
             file_callback (file, user_data);
 
-        g_clear_object (&in);
-        in = G_INPUT_STREAM (g_file_read (file->file, cancellable, error));
+        in = gcab_file_get_input_stream (file, cancellable, error);
         if (in == NULL)
-            goto end;
+            return FALSE;
 
         while ((len = g_input_stream_read (in,
                                            &data[offset], DATABLOCKSIZE - offset,
                                            cancellable, error)) == (DATABLOCKSIZE - offset)) {
             if (!cdata_write (&block, dstream, folder.typecomp, data, DATABLOCKSIZE, &written, cancellable, error))
-                goto end;
-            header.size += written;
+                return FALSE;
+            cheader->size += written;
             offset = 0;
         }
 
         if (len == -1)
-            goto end;
+            return FALSE;
 
         offset += len;
     }
     if (offset != 0) {
         if (!cdata_write (&block, dstream, folder.typecomp, data, offset, &written, cancellable, error))
-            goto end;
-        header.size += written;
+            return FALSE;
+        cheader->size += written;
     }
 
     if (!g_seekable_seek (G_SEEKABLE (out), 0,
                           G_SEEK_SET, cancellable, error))
-        goto end;
+        return FALSE;
 
-    header.nfiles = nfiles;
-    header.size += header.offsetfiles + nfiles * 16; /* 1st part cfile struct = 16 bytes */
-    header.size += sumstr;
+    cheader->nfiles = nfiles;
+    cheader->size += cheader->offsetfiles + nfiles * 16; /* 1st part cfile struct = 16 bytes */
+    cheader->size += sumstr;
 
-    if (!cheader_write (&header, dstream, cancellable, error))
-        goto end;
+    if (!cheader_write (cheader, dstream, cancellable, error))
+        return FALSE;
 
     if (!cfolder_write (&folder, dstream, cancellable, error))
-        goto end;
+        return FALSE;
 
-    for (l = files; l != NULL; l = l->next) {
-        file = GCAB_FILE (l->data);
-        file->cfile.uoffset = prevf ? prevf->uoffset + prevf->usize : 0;
-        prevf = &file->cfile;
+    for (GSList *l = files; l != NULL; l = l->next) {
+        GCabFile *file = GCAB_FILE (l->data);
+        gcab_file_set_uoffset (file, prevf ? gcab_file_get_uoffset (prevf) + gcab_file_get_usize (prevf) : 0);
+        prevf = file;
 
         /* automatically set flag if UTF-8 encoding */
-        if (!g_str_is_ascii (file->cfile.name))
-            file->cfile.fattr |= GCAB_FILE_ATTRIBUTE_NAME_IS_UTF;
+        if (!g_str_is_ascii (gcab_file_get_name (file)))
+            gcab_file_add_attribute (file, GCAB_FILE_ATTRIBUTE_NAME_IS_UTF);
 
-        if (!cfile_write (&file->cfile, dstream, cancellable, error))
-            goto end;
+        if (!cfile_write (gcab_file_get_cfile (file), dstream, cancellable, error))
+            return FALSE;
     }
 
-    success = TRUE;
+    /* replace the cached copy */
+    if (self->cheader != NULL)
+        cheader_free (self->cheader);
+    self->cheader = g_steal_pointer (&cheader);
 
-end:
-    g_clear_object (&dstream);
-    g_clear_object (&in);
-    g_slist_free (files);
-
-    return success;
+    return TRUE;
 }
 
 /**
@@ -411,75 +437,72 @@ gcab_cabinet_load (GCabCabinet *self,
     g_return_val_if_fail (self->folders->len == 0, FALSE);
     g_return_val_if_fail (self->stream == NULL, FALSE);
 
-    self->stream = g_object_ref (stream);
-
-    gboolean success = FALSE;
-    cheader_t cheader;
-    GDataInputStream *in = g_data_input_stream_new (stream);
+    g_autoptr(cheader_t) cheader = NULL;
+    g_autoptr(GDataInputStream) in = g_data_input_stream_new (stream);
     g_filter_input_stream_set_close_base_stream (G_FILTER_INPUT_STREAM (in), FALSE);
     g_data_input_stream_set_byte_order (in, G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN);
 
-    GPtrArray *folders = self->folders;
+    cheader = g_new0 (cheader_t, 1);
+    if (!cheader_read (cheader, in, cancellable, error))
+        return FALSE;
 
-    if (!cheader_read (&cheader, in, cancellable, error))
-        goto end;
-
-    if (cheader.reserved)
-        g_object_set (self, "reserved",
-                      g_byte_array_new_take (cheader.reserved, cheader.res_header),
-                      NULL);
-
-    for (guint i = 0; i < cheader.nfolders; i++) {
-        cfolder_t cfolder = { 0, };
-        if (!cfolder_read (&cfolder, cheader.res_folder, in, cancellable, error))
-            goto end;
-
-        GCabFolder *folder = gcab_folder_new_with_cfolder (&cfolder, stream);
-        if (cfolder.reserved)
-            g_object_set (folder, "reserved",
-                          g_byte_array_new_take (cfolder.reserved, cheader.res_folder),
-                          NULL);
-        g_ptr_array_add (folders, folder);
-        cfolder.reserved = NULL;
+    if (cheader->reserved != NULL) {
+        g_autoptr(GByteArray) blob = NULL;
+        blob = g_byte_array_new_take (cheader->reserved, cheader->res_header);
+        g_object_set (self, "reserved", blob, NULL);
+        cheader->reserved = NULL;
     }
 
-    for (guint i = 0; i < cheader.nfiles; i++) {
-        cfile_t cfile = { 0, };
-        if (!cfile_read (&cfile, in, cancellable, error))
-            goto end;
+    for (guint i = 0; i < cheader->nfolders; i++) {
+        g_autoptr(cfolder_t) cfolder = g_new0 (cfolder_t, 1);
+        g_autoptr(GByteArray) blob = NULL;
+        if (!cfolder_read (cfolder, cheader->res_folder, in, cancellable, error))
+            return FALSE;
 
-        if (cfile.index >= folders->len) {
-            g_set_error (error, GCAB_ERROR, GCAB_ERROR_FORMAT,
-                         "Invalid folder index");
-            goto end;
+        /* steal this inelegantly */
+        if (cfolder->reserved != NULL) {
+            blob = g_byte_array_new_take (cfolder->reserved, cheader->res_folder);
+            cfolder->reserved = NULL;
         }
 
-        GCabFolder *folder = g_ptr_array_index (folders, cfile.index);
+        GCabFolder *folder = gcab_folder_new_steal_cfolder (&cfolder);
+        if (blob != NULL)
+            g_object_set (folder, "reserved", blob, NULL);
+        g_ptr_array_add (self->folders, folder);
+    }
+
+    for (guint i = 0; i < cheader->nfiles; i++) {
+        g_autoptr(cfile_t) cfile = g_new0 (cfile_t, 1);
+        if (!cfile_read (cfile, in, cancellable, error))
+            return FALSE;
+
+        if (cfile->index >= self->folders->len) {
+            g_set_error (error, GCAB_ERROR, GCAB_ERROR_FORMAT,
+                         "Invalid folder index");
+            return FALSE;
+        }
+
+        GCabFolder *folder = g_ptr_array_index (self->folders, cfile->index);
         if (folder == NULL) {
             g_set_error (error, GCAB_ERROR, GCAB_ERROR_FORMAT,
                          "Invalid folder pointer");
-            goto end;
+            return FALSE;
         }
 
-        GCabFile *file = gcab_file_new_with_cfile (&cfile);
-        if (!gcab_folder_add_file (folder, file, FALSE, cancellable, error)) {
-            g_object_unref (file);
-            goto end;
-        }
+        g_autoptr(GCabFile) file = gcab_file_new_steal_cfile (&cfile);
+        if (!gcab_folder_add_file (folder, file, FALSE, cancellable, error))
+            return FALSE;
     }
 
-    success = TRUE;
-
-end:
-    if (in)
-        g_object_unref (in);
-    return success;
+    self->stream = g_object_ref (stream);
+    self->cheader = g_steal_pointer (&cheader);
+    return TRUE;
 }
 
 /**
  * gcab_cabinet_extract:
  * @cabinet: a #GCabCabinet
- * @path: the path to extract files
+ * @path: (allow-none): the path to extract files
  * @file_callback: (allow-none) (scope call) (closure user_data): an optional #GCabFile callback,
  *     return %FALSE to filter out or skip files.
  * @progress_callback: (allow-none) (scope call) (closure user_data): a progress callback
@@ -490,6 +513,9 @@ end:
  *
  * Extract files to given path.
  *
+ * If @path is NULL then the files are decompressed to memory blobs stored on
+ * each #GCabFile.
+ *
  * Returns: %TRUE on success.
  **/
 gboolean
@@ -501,24 +527,32 @@ gcab_cabinet_extract (GCabCabinet *self,
                       GCancellable *cancellable,
                       GError **error)
 {
-    gboolean success = TRUE;
-
     g_return_val_if_fail (GCAB_IS_CABINET (self), FALSE);
-    g_return_val_if_fail (G_IS_FILE (path), FALSE);
+    g_return_val_if_fail (!path || G_IS_FILE (path), FALSE);
     g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), FALSE);
     g_return_val_if_fail (!error || *error == NULL, FALSE);
 
+    /* never loaded from a stream */
+    if (self->cheader == NULL) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
+                     "Cabinet has not been loaded");
+        return FALSE;
+    }
+
+    g_autoptr(GDataInputStream) data = g_data_input_stream_new (self->stream);
+    g_data_input_stream_set_byte_order (data, G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN);
+    g_filter_input_stream_set_close_base_stream (G_FILTER_INPUT_STREAM (data), FALSE);
+
     for (guint i = 0; i < self->folders->len; ++i) {
         GCabFolder *folder = g_ptr_array_index (self->folders, i);
-        if (!gcab_folder_extract (folder, path, self->cheader.res_data,
+        if (!gcab_folder_extract (folder, data, path, self->cheader->res_data,
                                   file_callback, progress_callback, user_data,
                                   cancellable, error)) {
-            success = FALSE;
-            break;
+            return FALSE;
         }
     }
 
-    return success;
+    return TRUE;
 }
 
 /**
@@ -557,8 +591,8 @@ gcab_cabinet_extract_simple (GCabCabinet *cabinet,
  * Lookup the cabinet authenticode signature if any.
  *
  * Since: 0.5
- * Returns: the array containing the PKCS#7 signed data or %NULL if
- * none found or error.
+ *
+ * Returns: the array containing the PKCS#7 signed data or %NULL on error.
  **/
 const GByteArray *
 gcab_cabinet_get_signature (GCabCabinet *self,
@@ -566,6 +600,7 @@ gcab_cabinet_get_signature (GCabCabinet *self,
                             GError **error)
 {
     const guint8 magic[] = { 0x00, 0x00, 0x10, 0x00 };
+    gssize sz;
     guint8 *reserved;
     guint32 offset;
     guint32 size;
@@ -577,15 +612,24 @@ gcab_cabinet_get_signature (GCabCabinet *self,
     if (self->signature)
         return self->signature;
 
-    if (!G_IS_SEEKABLE (self->stream))
+    if (!G_IS_SEEKABLE (self->stream)) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_NOT_SUPPORTED,
+                     "Cabinet stream is not seekable");
         return NULL;
+    }
 
-    if (!self->reserved || self->reserved->len != 20)
+    if (!self->reserved || self->reserved->len != 20) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
+                     "Cabinet has no reserved area");
         return NULL;
+    }
 
     reserved = self->reserved->data;
-    if (memcmp (reserved, magic, sizeof (magic)) != 0)
+    if (memcmp (reserved, magic, sizeof (magic)) != 0) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FORMAT,
+                     "Cabinet reserved magic was not correct");
         return NULL;
+    }
 
     offset = GCAB_READ_UINT32_LE (reserved + 4);
     size = GCAB_READ_UINT32_LE (reserved + 8);
@@ -595,12 +639,24 @@ gcab_cabinet_get_signature (GCabCabinet *self,
     self->signature = g_byte_array_sized_new (size);
     g_byte_array_set_size (self->signature, size);
 
-    if (!g_seekable_seek (G_SEEKABLE (self->stream), offset, G_SEEK_SET, cancellable, error))
+    if (!g_seekable_seek (G_SEEKABLE (self->stream), offset, G_SEEK_SET, cancellable, error)) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_INVALID_DATA,
+                     "Cannot seek to reserved area");
         return NULL;
+    }
 
-    if (g_input_stream_read (self->stream, self->signature->data, self->signature->len,
-                             cancellable, error) != self->signature->len)
+    sz = g_input_stream_read (self->stream,
+                              self->signature->data, self->signature->len,
+                              cancellable, error);
+    if (sz < 0) {
+        g_prefix_error (error, "Failed to read signature from stream: ");
         return NULL;
+    }
+    if (sz != self->signature->len) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
+                     "Failed to read correct size signature from stream: ");
+        return NULL;
+    }
 
     return self->signature;
 }
