@@ -29,12 +29,13 @@
 
 G_BEGIN_DECLS
 
+/**
+ * GCabCabinet:
+ *
+ * An opaque object holding a Cabinet file reference.
+ **/
 #define GCAB_TYPE_CABINET                (gcab_cabinet_get_type ())
-#define GCAB_CABINET(cabinet)            (G_TYPE_CHECK_INSTANCE_CAST ((cabinet), GCAB_TYPE_CABINET, GCabCabinet))
-#define GCAB_CABINET_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), GCAB_TYPE_CABINET, GCabCabinetClass))
-#define GCAB_IS_CABINET(cabinet)         (G_TYPE_CHECK_INSTANCE_TYPE ((cabinet), GCAB_TYPE_CABINET))
-#define GCAB_IS_CABINET_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), GCAB_TYPE_CABINET))
-#define GCAB_CABINET_GET_CLASS(cabinet)  (G_TYPE_INSTANCE_GET_CLASS ((cabinet), GCAB_TYPE_CABINET, GCabCabinetClass))
+G_DECLARE_FINAL_TYPE(GCabCabinet, gcab_cabinet, GCAB, CABINET, GObject)
 
 /**
  * GCAB_ERROR:
@@ -47,8 +48,10 @@ GQuark gcab_error_quark (void);
 
 /**
  * GCabError:
- * @GCAB_ERROR_FORMAT: The given file is not of Cabinet format.
- * @GCAB_ERROR_FAILED: General function failure.
+ * @GCAB_ERROR_FORMAT:          The given file is not of Cabinet format.
+ * @GCAB_ERROR_FAILED:          General function failure.
+ * @GCAB_ERROR_NOT_SUPPORTED:   Action or format is not supported
+ * @GCAB_ERROR_INVALID_DATA:    Data stream was invalid
  *
  * The various errors triggered by the GCab functions.
  **/
@@ -56,19 +59,10 @@ typedef enum GCabError
 {
     GCAB_ERROR_FORMAT,
     GCAB_ERROR_FAILED,
+    GCAB_ERROR_NOT_SUPPORTED,   /* Since: 1.0 */
+    GCAB_ERROR_INVALID_DATA,    /* Since: 1.0 */
 } GCabError;
 
-typedef struct _GCabCabinetClass GCabCabinetClass;
-
-/**
- * GCabCabinet:
- *
- * An opaque object holding a Cabinet file reference.
- **/
-typedef struct _GCabCabinet GCabCabinet;
-
-GType gcab_cabinet_get_type (void) G_GNUC_CONST;
-
 GCabCabinet *      gcab_cabinet_new           (void);
 gboolean           gcab_cabinet_load          (GCabCabinet *cabinet,
                                                GInputStream *stream,
@@ -76,6 +70,7 @@ gboolean           gcab_cabinet_load          (GCabCabinet *cabinet,
                                                GError **error);
 
 GPtrArray *        gcab_cabinet_get_folders   (GCabCabinet *cabinet);
+guint32            gcab_cabinet_get_size      (GCabCabinet *cabinet);
 
 gboolean           gcab_cabinet_add_folder    (GCabCabinet *cabinet,
                                                GCabFolder *folder,
